package org.coin.util.image.filter;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.Rectangle;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.imageio.ImageIO;

import org.coin.util.image.CoinImageUtil;

public class ImageFilterOp {
	
	public static BufferedImage getBlackWhite(
			BufferedImage image) 
	throws IOException
	{
		int w = image.getWidth();
		int h = image.getHeight();
		BufferedImage imageBW = new BufferedImage(w, h, BufferedImage.TYPE_BYTE_BINARY);
		Graphics2D g = imageBW.createGraphics();
		g.drawRenderedImage(image, null);
		g.dispose();

		return  imageBW;
	}
	
	public static BufferedImage getThresholdImage(
			BufferedImage image)
	throws IOException 
	{
		BufferedImage out = CoinImageUtil.copy(image, BufferedImage.TYPE_INT_ARGB );
		float fSaturationMin = 0.10f;
		float fBrightnessMin = 0.80f;
		
		ImageFilterOp.computeBlackAndWhite(image, out, fSaturationMin, fBrightnessMin);
		
		return out;
	}
	
	public static BufferedImage getAllAreaAndDisplayOnImage(
			BufferedImage image,
			int iMaxBlankPixel )
	throws IOException 
	{
		BufferedImage out = CoinImageUtil.copy(image, BufferedImage.TYPE_INT_ARGB );
		Vector<Rectangle> vArea = ImageFilterOp.getAllAreaByColor(image, out, 0, 0, 0, iMaxBlankPixel);

		Graphics2D gc = out.createGraphics();
		gc.setColor(new Color(0f, 0f, 1f));

		for(Rectangle rectangle : vArea){
			gc.draw(rectangle);
		}
		
		return out;
	}
	
	public static Vector<Rectangle> getAllAreaByColor(
			BufferedImage image,
			int[] iColor,
			int iMaxBlankPixel,
			double dDensityMin,
			int iCountPointMin)
	throws IOException 
	{
		//BufferedImage imageTemp = CoinImageUtil.copy(image, BufferedImage.TYPE_INT_ARGB );
		Vector<Rectangle> vArea 
		= ImageFilterOp.getAllAreaByColor(image, null, iColor, iMaxBlankPixel);

		vArea = ImageFilterOp.filterAllAreaByMinDensity(
				image, 
				vArea, 
				iColor, 
				dDensityMin,
				iCountPointMin);

		return vArea;
	}
	
	public static BufferedImage filterByColor(
			BufferedImage image,
			int[] iColor ,
			int[] iColorClean )
	throws IOException 
	{
		BufferedImage imageTemp = CoinImageUtil.copy(image, BufferedImage.TYPE_INT_ARGB );
		BufferedImage imageOut = CoinImageUtil.copy(image, BufferedImage.TYPE_INT_ARGB );

		
		ImageFilterOp.removeBlackColor(image, imageTemp);
		ImageFilterOp.cleanByTint(imageTemp, imageOut, iColor, iColorClean);
		

		return imageOut ;
	}
	
	
	public static void updateWhiteTransparency(
			String sFileIn,
			String sFileOut) 
	throws IOException 
	{
		BufferedImage in = ImageIO.read( new File( sFileIn));

		BufferedImage out = getImageWhiteTransparency(in);
		ImageIO.write(out, "png",  new File(sFileOut));
	}	

	
	public static BufferedImage getImageWhiteTransparency(
			String sFileIn) 
	throws IOException 
	{
		BufferedImage in = ImageIO.read( new File( sFileIn));
		return getImageWhiteTransparency(in);
	}

	public static BufferedImage getImageWhiteTransparency(
			BufferedImage in) 
	throws IOException 
	{
		Graphics2D gin = in.createGraphics();
		GraphicsConfiguration gc = gin.getDeviceConfiguration();
		gin.dispose();

		BufferedImage out =
			gc.createCompatibleImage(in.getWidth(), in.getHeight(), Transparency.BITMASK);

		updateWhiteTransparency(in, out);

		return out;
	}

	public static void updateWhiteTransparency(
			BufferedImage in,
			BufferedImage out) 
	throws IOException 
	{
		updateWhiteTransparency(in, out, 250, 255);
	}
	
	public static void updateWhiteTransparency(
			BufferedImage in,
			BufferedImage out,
			int iBottom ,
			int iBlankTolerance) 
	throws IOException 
	{
		int w = in.getWidth();
		int h = in.getHeight();
		int pixel;

		for (int x = 0; x < w; x++) {
			for (int y = 0; y < h; y++) {
				pixel = in.getRGB(x, y);

				int alpha = ((pixel >> 24 ) & 0xFF);
				int red = ((pixel >> 16 ) & 0xFF);
				int green = ((pixel >> 8 ) & 0xFF);
				int blue = (pixel & 0xFF);

				
				/**
				 * il faut gerer le bruit
				 */
				
				if( (red +  green + blue > iBottom *3 )
				&& (Math.abs(red - green) < iBlankTolerance)
				&& (Math.abs(red - blue) < iBlankTolerance)
				&& (Math.abs(green - blue) < iBlankTolerance)
				)
				{
					/**
					 * its quite to pure white color, then apply filter
					 */
					//float[] hsb = new float[3];
					//Color.RGBtoHSB(red, green, blue, hsb);
					//alpha = (int) (255 - ((red * green * blue) / (255 * 255) ) * dTransparencyRatio);
					double d = ( red * 0.25 + green * 0.65 + blue * 0.1) ;
					//alpha = (int) (255 - d * d / 255);
					alpha = (int) (255 - d );
					pixel = pixel & 0x00FFFFFF;
					pixel = pixel | (alpha <<24 );
				} else {
					alpha = 255;
					pixel = pixel & 0x00FFFFFF;
					pixel = pixel | (alpha <<24 );
				}
				out.setRGB(x, y, pixel);
			}
		}
	}
	public static BufferedImage removeBlackColor(
			BufferedImage image) 
	throws IOException 
	{
		BufferedImage out = CoinImageUtil.copy(image, BufferedImage.TYPE_INT_ARGB );
		removeBlackColor(image, out);
		return out;
	}
	
	public static void removeBlackColor(
			BufferedImage in,
			BufferedImage out) 
	throws IOException 
	{
		int w = in.getWidth();
		int h = in.getHeight();
		int pixel;

		for (int x = 0; x < w; x++) {
			for (int y = 0; y < h; y++) {
				pixel = in.getRGB(x, y);

				int alpha = ((pixel >>24 ) & 0xFF);
				int red = ((pixel >>16 ) & 0xFF);
				int green = ((pixel >>8 ) & 0xFF);
				int blue = (pixel & 0xFF);

				
				/**
				 * il faut gerer le bruit
				 */
				int iBottom = 255;
				int iBlackTolerance = 20;
				
				if( (red +  green + blue < iBottom *3 )
				&& (Math.abs(red - green) < iBlackTolerance)
				&& (Math.abs(red - blue) < iBlackTolerance)
				&& (Math.abs(green - blue) < iBlackTolerance)
				)
				{
					red = 255;
					green = 255;
					blue = 255;
				} else{
					/*red = 255;
					green = 255;
					blue = 255;*/
				}
				pixel = pixel & 0x00000000;
				pixel = pixel | (alpha <<24 );
				pixel = pixel | (red <<16 );
				pixel = pixel | (green <<8 );
				pixel = pixel | (blue  );

				out.setRGB(x, y, pixel);
			}
		}
	}
	
	public static void computeBlackAndWhite(
			BufferedImage in,
			BufferedImage out,
			float fSaturationMin,
			float fBrightnessMin) 
	throws IOException 
	{
		int w = in.getWidth();
		int h = in.getHeight();
		int pixel;
		float[] hsb = new float[3];

		for (int x = 0; x < w; x++) {
			for (int y = 0; y < h; y++) {
				pixel = in.getRGB(x, y);

				int alpha = ((pixel >>24 ) & 0xFF);
				int red = ((pixel >>16 ) & 0xFF);
				int green = ((pixel >>8 ) & 0xFF);
				int blue = (pixel & 0xFF);

				Color.RGBtoHSB(red, green, blue, hsb);
				
				if( hsb[2] < fBrightnessMin
				|| (hsb[2] >= fBrightnessMin &&  hsb[1] >= fSaturationMin)
				// (red +  green + blue > iThreshold *3 )
				)
				{
					red = 0;
					green = 0;
					blue = 0;
				} else {
					red = 255;
					green = 255;
					blue = 255;
				}
				pixel = pixel & 0x00000000;
				pixel = pixel | (alpha <<24 );
				pixel = pixel | (red <<16 );
				pixel = pixel | (green <<8 );
				pixel = pixel | (blue  );

				out.setRGB(x, y, pixel);
			}
		}
	}
	
	
	public static BufferedImage getTintImage(
			BufferedImage image)
	throws IOException 
	{
		int iTintByPassedMin = 170;
		int iTintByPassedMax = 250;
		int iWidth = 360;
		int iHeight = 200;
		
		long[] tint = ImageFilterOp.getTint(image, iTintByPassedMin, iTintByPassedMax);
		Graphics2D gin = image.createGraphics();
		GraphicsConfiguration gc = gin.getDeviceConfiguration();
		gin.dispose();

		BufferedImage outTint = 
			gc.createCompatibleImage(iWidth, iHeight);

		ImageFilterOp.getTintImage(tint, outTint);
		
		return outTint;
	}


	public static void cleanByTint(
			BufferedImage in,
			BufferedImage out,
			int[] colorAndTolerance) 
	throws IOException 
	{
		cleanByTint(
				in, 
				out, 
				colorAndTolerance[0], 
				colorAndTolerance[1], 
				colorAndTolerance[2], 
				colorAndTolerance[3],
				colorAndTolerance[0], 
				colorAndTolerance[1], 
				colorAndTolerance[2]);
	}
	
	public static void cleanByTint(
			BufferedImage in,
			BufferedImage out,
			int[] colorAndTolerance,
			int[] colorClean) 
	throws IOException 
	{
		cleanByTint(
				in, 
				out, 
				colorAndTolerance[0], 
				colorAndTolerance[1], 
				colorAndTolerance[2], 
				colorAndTolerance[3],
				colorClean[0], 
				colorClean[1], 
				colorClean[2]);
	}
	
	public static void cleanByTint(
			BufferedImage in,
			BufferedImage out,
			int iRed,
			int iGreen,
			int iBlue,
			int iTolerance,
			int iRedClean,
			int iGreenClean,
			int iBlueClean) 
	throws IOException 
	{
		int w = in.getWidth();
		int h = in.getHeight();
		int pixel;
		float[] hsb = new float[3];
		Color.RGBtoHSB(iRed, iGreen, iBlue, hsb);
		System.out.println( " r=" + iRed + "\tg=" + iGreen + "\tb=" + iBlue );
		System.out.println("inital tint : " 
					+ (hsb[0] * 360) + " / " 
					+ hsb[1] + " / "
					+ hsb[2] );
		
		
		
		int iTint = (int)(hsb[0] * 360) ;
		int colorized = 0;

		for (int x = 0; x < w; x++) {
			for (int y = 0; y < h; y++) {
				pixel = in.getRGB(x, y);

				int alpha = ((pixel >>24 ) & 0xFF);
				int red = ((pixel >>16 ) & 0xFF);
				int green = ((pixel >>8 ) & 0xFF);
				int blue = (pixel & 0xFF);

				Color.RGBtoHSB(red, green, blue, hsb);
				
				int diff = (int) Math.abs( (hsb[0] * 360) - iTint) ;
				
				if(diff < iTolerance )
				{
					red = iRedClean;
					green = iGreenClean;
					blue = iBlueClean;
					colorized++;
				} else {
					red = 255;
					green = 255;
					blue = 255;
				}
				pixel = pixel & 0x00000000;
				pixel = pixel | (alpha <<24 );
				pixel = pixel | (red <<16 );
				pixel = pixel | (green <<8 );
				pixel = pixel | (blue  );

				out.setRGB(x, y, pixel);
			}
		}
		
	}
	
	
	
	public static int[] getAreaByTint(
			BufferedImage in,
			BufferedImage out,
			int[] color,
			int iTolerance) 
	throws IOException 
	{
		return getAreaByTint(in, out, color[0], color[1], color[2], iTolerance);
	}
	
	public static int[] getAreaByTint(
			BufferedImage in,
			BufferedImage out,
			int iRed,
			int iGreen,
			int iBlue,
			int iTolerance) 
	throws IOException 
	{
		int w = in.getWidth();
		int h = in.getHeight();
		int pixel;
		int[] area = new int[4];
		float[] hsb = new float[3];
		Color.RGBtoHSB(iRed, iGreen, iBlue, hsb);
		System.out.println( " r=" + iRed + "\tg=" + iGreen + "\tb=" + iBlue );
		System.out.println("inital tint : " 
					+ (hsb[0] * 360) + " / " 
					+ hsb[1] + " / "
					+ hsb[2] );
		
		
		
		int iTint = (int)(hsb[0] * 360) ;
		int colorized = 0;
		/**
		 * area
		 */
		int x1 = w;
		int y1 = h;
		int x2 = 0;
		int y2 = 0;
		int iMaxBlankLine = 50;
		
		for (int x = 0; x < w; x++) {
			for (int y = 0; y < h; y++) {
				pixel = in.getRGB(x, y);

				int alpha = ((pixel >>24 ) & 0xFF);
				int red = ((pixel >>16 ) & 0xFF);
				int green = ((pixel >>8 ) & 0xFF);
				int blue = (pixel & 0xFF);

				Color.RGBtoHSB(red, green, blue, hsb);
				
				int diff = (int) Math.abs( (hsb[0] * 360) - iTint) ;
				
				if(diff < iTolerance )
				{
					/*
					System.out.println("OK tint : " 
							+ (hsv[0] * 360) + " / " 
							+ hsv[1] + " / "
							+ hsv[2] );
					
					red = iRed;
					green = iGreen;
					blue = iBlue;
					*/
					
					/**
					 * analyse blank line
					 */
					if(y2 > 0)
					{
						if(y2 + iMaxBlankLine < y)
						{
							/**
							 * not on the area : out of bounds
							 */
							continue;
						}
					}
					
					if(x < x1)
					{
						x1 = x;
					}
					if(y < y1)
					{
						y1 = y;
					}
					if(x > x2)
					{
						x2 = x;
					}
					if(y > y2)
					{
						y2 = y;
					}
					
					red = 0;
					green = 0;
					blue = 0;
					
					colorized++;
				} 
				pixel = pixel & 0x00000000;
				pixel = pixel | (alpha <<24 );
				pixel = pixel | (red <<16 );
				pixel = pixel | (green <<8 );
				pixel = pixel | (blue  );

				out.setRGB(x, y, pixel);
			}
		}
		
		System.out.println("colorized : " + colorized);
		System.out.println(
				"x1=" + x1 
				+ " y1=" + y1
				+ " x2=" + x2
				+ " y2=" + y2
				);
		
		Graphics2D gc = in.createGraphics();
		gc.setColor(new Color(1f, 0f, 0f));
		gc.drawRect(x1, y1, x2-x1, y2-y1);
		gc.drawLine(x1, y1, x2, y2);
		
		return area;
	}
	
	
	public static Vector<Rectangle> filterAllAreaByMinDensity(
			BufferedImage in,
			Vector<Rectangle> vArea,
			int[] color,
			double dDensityMin,
			int iCountPointMin) 
	throws IOException 
	{
		return filterAllAreaByMinDensity(
				in, 
				vArea, 
				color[0], 
				color[1], 
				color[2], 
				dDensityMin,
				iCountPointMin);
	}
	
	public static Vector<Rectangle> filterAllAreaByMinDensity(
			BufferedImage in,
			Vector<Rectangle> vArea,
			int iRed,
			int iGreen,
			int iBlue,
			double dDensityMin,
			int iCountPointMin ) 
	throws IOException 
	{
		Vector<Rectangle> vAreaOut = new Vector<Rectangle>(1000);
		int pixel;

		for (Rectangle rectangle : vArea) {
			/**
			 * clean rectangle
			 */
			if(rectangle.x < 0) rectangle.x = 0;
			if(rectangle.y < 0) rectangle.y = 0;
			
			/**
			 * compute density
			 */
			double dCountPoint = 0;
			double dCountPointInRectangle = rectangle.height *  rectangle.width;
			for (int x = 0; x < rectangle.width; x++) {
				for (int y = 0; y < rectangle.height; y++) {
					pixel = in.getRGB(rectangle.x + x, rectangle.y + y);
					int red = ((pixel >>16 ) & 0xFF);
					int green = ((pixel >>8 ) & 0xFF);
					int blue = (pixel & 0xFF);

					if(red == iRed
					&& green == iGreen
					&& blue == iBlue )
					{
						dCountPoint++;
					}
				}
			}			
			
			double dDensity = dCountPoint / dCountPointInRectangle;
			System.out.println("density : " + dDensity 
					+ " dCountPoint=" + dCountPoint
					+ " dCountPointInRectangle=" + dCountPointInRectangle
					+ " for " + rectangle);
			
			if(dDensityMin <= dDensity){
				if(iCountPointMin <= dCountPoint )
				{
					vAreaOut.add(rectangle);
				}
			}
		}
		
		return vAreaOut;
	}
	
	public static Vector<Rectangle> getAllAreaByColor(
			BufferedImage in,
			BufferedImage out,
			int[] color,
			int iMaxBlankPixel) 
	throws IOException {
		
		return getAllAreaByColor(
				in, 
				out, 
				color[0], 
				color[1], 
				color[2], 
				iMaxBlankPixel);
	}
	
	public static Vector<Rectangle> getAllAreaByColor(
			BufferedImage in,
			BufferedImage out,
			int iRed,
			int iGreen,
			int iBlue,
			int iMaxBlankPixel) 
	throws IOException 
	{
		int w = in.getWidth();
		int h = in.getHeight();
		int pixel;
		Vector<Rectangle> vArea = new Vector<Rectangle>(1000);

		Graphics2D gc = null;
		
		if(out != null){
			gc = out.createGraphics();
			gc.setColor(new Color(1f, 0f, 0f));
		}
		
		//int iCount = 1500;

		int iMaxBlankPixelDouble = iMaxBlankPixel * 2;
		for (int x = 0; x < w; x++) {
			for (int y = 0; y < h; y++) {
				pixel = in.getRGB(x, y);
				int alpha = ((pixel >>24 ) & 0xFF);
				int red = ((pixel >>16 ) & 0xFF);
				int green = ((pixel >>8 ) & 0xFF);
				int blue = (pixel & 0xFF);

				
				if(red == iRed
				&& green == iGreen
				&& blue == iBlue )
				{
					Rectangle rect = new Rectangle(
							x - iMaxBlankPixel, 
							y - iMaxBlankPixel, 
							iMaxBlankPixelDouble,
							iMaxBlankPixelDouble);
					
					
					boolean bInArea = false; 
					for(Rectangle rectangle : vArea){
						if(rectangle.contains(x, y))
						{
							rectangle.add(rect);
							//System.out.println(rectangle + " contains " + " x=" + x + " y=" + y );
							bInArea = true;
							break;
						}
					}
					
					if(bInArea )
					{
						continue;
					}

					
					
					pixel = pixel & 0x00000000;
					pixel = pixel | (alpha <<24 );
					pixel = pixel | (0 <<16 );
					pixel = pixel | (255 <<8 );
					pixel = pixel | (0  );

					
					//System.out.println("found x=" + x + " y=" + y);

					bInArea = false;
					for(Rectangle rectangle : vArea){
						Rectangle intersection = rectangle.intersection(rect);
						if(intersection.width > 0 && intersection.height > 0)
						{
							bInArea = true;
							rectangle.add(rect);
							
							break;
						}
					}
					
					if(!bInArea) vArea.add(rect);
					
					while(bInArea)
					{
						Rectangle rectToRemove = null;
						bInArea = false;
						for(Rectangle rectangle : vArea){
							
							for(Rectangle rec2 : vArea)
							{
								if(rec2 == rectangle) continue;
								
								Rectangle intersection = rectangle.intersection(rec2);
								if(intersection.width > 0 && intersection.height > 0)
								{
									System.out.println(rectangle + " intersect " + rec2);
									bInArea = true;
									rectangle.add(rec2);
									rectToRemove = rec2;
									break;
								}
								if(bInArea) break;
							}
						}
						if(rectToRemove != null) vArea.remove(rectToRemove);
					}
					
					
					if(out != null){
						out.setRGB(x, y, pixel);
						gc.draw(rect);
					}
					
					
					//if(iCount-- == 0) return vArea;
				}
				

			}
		}
		
		
		return vArea;
	}
	
	

	
	public static long[] getTint(
			BufferedImage in,
			int iTintByPassedMin,
			int iTintByPassedMax)
	throws IOException 
	{
		int w = in.getWidth();
		int h = in.getHeight();
		int pixel;
		float[] hsv = new float[3];
		long[] tint = new long[361];
		
		for (int x = 0; x < w; x++) {
			for (int y = 0; y < h; y++) {
				pixel = in.getRGB(x, y);

				//int alpha = ((pixel >>24 ) & 0xFF);
				int red = ((pixel >>16 ) & 0xFF);
				int green = ((pixel >>8 ) & 0xFF);
				int blue = (pixel & 0xFF);

				Color.RGBtoHSB(red, green, blue, hsv);
				int color = (int)(hsv[0] * 360);
				if(color > iTintByPassedMin && color < iTintByPassedMax) continue;
				
				tint[color ]++;
			}
		}
		return tint;
	}

	public static void getTintImage(
			long[] tint,
			BufferedImage out)
	throws IOException 
	{
		/**
		 * compute max
		 */
		long lTintCountMax = 0;
		for (int i = 0; i < tint.length; i++) {
			long l = tint[i];
			if(lTintCountMax < l) lTintCountMax = l;
		}
		
		/*
		for (int i = 0; i < tint.length; i++) {
			long l = tint[i];
			System.out.println(i + ": " + l);
		}
		*/
		
		int w = out.getWidth();
		int h = out.getHeight();
		int pixel;

		/**
		 * set to white
		 */
		for (int x = 0; x < w; x++) {
			for (int y = 0; y < h ; y++) {
				pixel = 0xFFFFFFFF;
				out.setRGB(x, y, pixel);
			}
		}
		
		for (int x = 0; x < w; x++) {
			int k = 
				(int)
				(Math.log( (tint[x] * h / lTintCountMax)) 
				* h
				/ Math.log(h ));
			
			k = 
				(int)
				(tint[x] * h / lTintCountMax);
			
			pixel = Color.HSBtoRGB( ((float)x/(float)360), 1, 1);
			int alpha = 255;
			int red = ((pixel >>16 ) & 0xFF);
			int green = ((pixel >>8 ) & 0xFF);
			int blue = (pixel & 0xFF);
			pixel = pixel & 0x00000000;
			pixel = pixel | (alpha <<24 );
			pixel = pixel | (red <<16 );
			pixel = pixel | (green <<8 );
			pixel = pixel | (blue  );
			
			for (int y = 0; y < k ; y++) {
				if((h-y) >= h) continue;
				out.setRGB(x, h - y, pixel);
			}
		}
	}
	
}
