package org.coin.util.image.shape;


import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.imageio.ImageIO;



public abstract class PdfImageShape extends PdfShape {
	
	
	public InputStream isImage;
	public BufferedImage biImage;
	public File fileImage;

	public String imageContentType = "image/jpg";
	public double imageRatio = 1d;
	public boolean forceBlackWhite = false;
	public boolean filterTransparency = false;
	
	public PdfImageShape() {
		init();
	}

	public void init() {
		this.shapeType = "image";
	}

	@Override
	public void draw(int pageNumber) throws IOException {
		BufferedImage bi = this.biImage;
		
		if(bi == null) {
			System.out.println("ERROR bi = null for " + this.getName());
			return;
		}
		if(getPageOrientation(pageNumber) == PageFormat.PORTRAIT) {
			// nothing
		} else {
			bi = rotateLeft(bi);
		}
	}
	
	@Override
	public String getShapeInnerXml() throws IOException {
		String innerXml = super.getShapeInnerXml();
		if(this.biImage != null)
		{
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			ImageIO.write(this.biImage, "jpg", baos);
			String sImageFileContentBase64 = getBase64String(baos.toByteArray());
			innerXml += "<image "
				+ " contentType=\"" + this.imageContentType + "\" "
				+ " encodingType=\"" + this.encodingType + "\" "
				+ " filename=\"" + this.fileImage.getName() + "\" "
				+ " >" 
				+ sImageFileContentBase64
				+ "</image>\n";
		}
		return innerXml;
	}

	public void setFileImage(File file) {
		this.fileImage = file;
	}

	public File getFileImage() {
		return this.fileImage;
	}

	public void loadFileImage(File fileImage) throws IOException {
		this.fileImage = fileImage;
		loadFileImage();
	}

	
	public void loadFileImage() throws IOException {
		this.biImage = ImageIO.read(this.fileImage);
	}

	public void loadFileImage(
			InputStream is)
	throws IOException {
		this.biImage = ImageIO.read(is);
		is.close();
	}

	
	public void setImage(BufferedImage image) {
		this.biImage = image;
	}
	
	public BufferedImage getImage() {
		return this.biImage;
	}
}
