package org.coin.util.image.shape;

import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Map;
import java.util.List;

import org.apache.commons.codec.binary.Base64;

public abstract class PdfShape {

	public static final String TAG_NAME_ROOT = "Sspv";
	public static final String TAG_NAME_DOCUMENT = "SspvDocument";
	public static final String TAG_NAME_TRANSFORMATION = "SspvTransformation";
	public static final String TAG_NAME_SHAPE = "SspvShape";
	
	protected Object document;
	public String encodingType = "Base64";

	protected String id;
	protected String shapeType;
	protected String name;
	protected String pageNumberList;
	protected double positionX;
	protected double positionY;
	protected double height;
	protected double width;
	protected double rotationAngle;
	protected Map<String, Object> mapObject ;

	protected boolean drawAreaLimit;
	protected boolean drawAreaFill;

	/**
	 * to init during the object creation
	 * @param mapObject
	 */
	public void setMapObject(Map<String, Object> mapObject) {
		this.mapObject = mapObject;
	}

	public Map<String, Object> getMapObject() {
		return this.mapObject;
	}

	public void setObject(Object obj) {
		mapObject.put(this.id, obj);
	}
	
	public void setObject(String sId, Object obj) {
		mapObject.put(sId, obj);
	}

	public Object getObject(String sId) {
		return mapObject.get(sId);
	}

	public Object getObject() {
		return mapObject.get(this.id);
	}


	
	public void setDrawAreaFill(boolean drawAreaFill) {
		this.drawAreaFill = drawAreaFill;
	}
	
	public void setDrawAreaLimit(boolean drawAreaLimit) {
		this.drawAreaLimit = drawAreaLimit;
	}
	
	public boolean isDrawAreaFill() {
		return this.drawAreaFill;
	}
	
	public boolean isDrawAreaLimit() {
		return this.drawAreaLimit;
	}
	
	public void setShapeType(String shapeType) {
		this.shapeType = shapeType;
	}
	
	public String getShapeType() {
		return shapeType;
	}
	
	/**
	 * draw shape on the document
	 * @throws IOException 
	 */
	public abstract void draw(int iPageNumber) throws IOException ;	
	
	/**
	 * 
	 * have to return value in this list :
	 *  java.awt.print.PageFormat.LANDSCAPE
	 *  java.awt.print.PageFormat.PORTRAIT
	 *  java.awt.print.PageFormat.REVERSE_LANDSCAPE
	 * 
	 * @param iPageNumber
	 * @return
	 * @throws IOException
	 */
	public abstract int getPageOrientation(int iPageNumber) throws IOException ;	
	
	

	public static List<Integer> getPageNumberCollection(String pageNumberList) {
		
		/*
		 * example : 1-4;6;12-56
		 */
		String[] pageRangeList = pageNumberList.split(";");
		List<Integer> pageList = new ArrayList<Integer>();
		
		for (int i = 0; i < pageRangeList.length; i++) {
			String[] pageRange = pageRangeList[i].split("-");
			if(pageRange.length == 1)
			{
				pageList.add(new Integer(pageRange[0]));
			} else {
				/**
				 * exemple : 1-4
				 */
				for (int j = Integer.parseInt(pageRange[0]); j <= Integer.parseInt(pageRange[1]); j++) {
					pageList.add(new Integer(j));
				}
			}
		}
		return pageList;
	}

	public void draw() throws IOException {
		
		List<Integer> pageList =getPageNumberCollection(this.pageNumberList);
		
		for (Integer pageNumber : pageList) {
			draw(pageNumber.intValue());
		}
	}
	
	public String getShapeXml() throws IOException {
		String sXml = "<" + TAG_NAME_SHAPE 
			+ " type=\""  + this.shapeType + "\""
			+ ">\n"
			+ getShapeInnerXml()
			+ "</" + TAG_NAME_SHAPE + ">\n";
		return sXml ;
	}	

	public String getShapeInnerXml() throws IOException {
		String sInnerXml 
		=  "<name>" + this.name + "</name>\n"
		+ "<id>" + this.id+ "</id>\n"
		+ "<pageNumberList>" + this.pageNumberList + "</pageNumberList>\n"
		+ "<positionX>" + this.positionX + "</positionX>\n"
		+ "<positionY>" + this.positionY + "</positionY>\n"
		+ "<width>" + this.width + "</width>\n"
		+ "<height>" + this.height + "</height>\n";
		return sInnerXml;
		
	
	}

	public Object getDocument() {
		return this.document;
	}

	public void setDocument(Object document) {
		this.document = document;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getId() {
		return this.id;
	}

	public void setId(String id) {
		this.id = id;
	}

	
	public String getPageNumberList() {
		return this.pageNumberList;
	}

	public void setPageNumberList(String pageNumberList) {
		this.pageNumberList = pageNumberList;
	}

	public void setPageNumberList(int pageNumber) {
		this.pageNumberList =  Integer.toString(pageNumber);
	}

	
	public double getPositionX() {
		return this.positionX;
	}

	public void setPositionX(double positionX ) {
		this.positionX = positionX;
	}

	public double getPositionY() {
		return this.positionY;
	}

	public void setPositionY(double positionY ) {
		this.positionY = positionY;
	}

	public double getHeight() {
		return this.height;
	}

	public void setHeight(double height) {
		this.height = height ;
	}
	
	public double getWidth() {
		return this.width;
	}

	public void setWidth(double width) {
		this.width = width ;
	}


	public double getRotationAngle() {
		return this.rotationAngle;
	}

	public void setRotationAngle(double rotationAngle) {
		this.rotationAngle = rotationAngle;
	}

	public void setBounds(
			double positionX,
			double positionY,
			double width,
			double height
			) {
		this.positionX = positionX ;
		this.positionY = positionY ;
		this.width = width ;
		this.height = height;
	}


	/**
	 * UTILS 
	 */
	public static String getBase64String(byte[] bytes) 
	throws UnsupportedEncodingException {
		return new String(Base64.encodeBase64(bytes), "ASCII");
	}
	
	
	/**
	 * UTILS 
	 */
	public static BufferedImage rotateLeft ( BufferedImage bi )  
	{  
		AffineTransform afLeft = AffineTransform.getRotateInstance (  
				Math.toRadians ( 270 )  ) ; 
		afLeft.translate ( bi.getWidth (  )  * -1,0 ) ; 
		AffineTransformOp lOp = new AffineTransformOp ( afLeft,null ) ; 
		BufferedImage dstbi = lOp.filter ( bi, null ) ; 
		return dstbi; 
	}  

}
