package org.coin.util.image.shape;

import java.awt.Font;
import java.awt.FontFormatException;
import java.io.File;
import java.io.IOException;

import org.coin.util.pdf.pdfbox.PdfBoxTextUtil;

import lb.service.crypto.util.FileUtilBasic;


public abstract class PdfTextShape extends PdfShape {
	public static final int TEXT_ALIGN_LEFT = 1;
	public static final int TEXT_ALIGN_CENTER = 2;
	public static final int TEXT_ALIGN_RIGHT = 3;
	public static final int TEXT_ALIGN_JUSTIFY = 4;

	/**
	 * put a specific mention
	 */
	public static final int TEXT_TRANSFORMATION_WRITE = 1;

	/**
	 * remove a part of text (build in Entreprise Edition)
	 */
	public static final int TEXT_TRANSFORMATION_REMOVE = 2;

	/**
	 * replace a part of text (build in Entreprise Edition)
	 */
	public static final int TEXT_TRANSFORMATION_REPLACE = 3;

	/**
	 * replace a part of text (build in Entreprise Edition)
	 */
	public static final int TEXT_TRANSFORMATION_APPEND = 4;
	
	protected String sTextEncoding="UTF-8";
	protected String sPdfEncoding = PdfBoxTextUtil.WIN_ANSI_ENCODING;
	protected String sText;
	protected Font font;
	protected File fileFont;
	protected double dFontSize = 12;
	
	public int iTextTransformation = TEXT_TRANSFORMATION_WRITE;
	public int iTextAlign = TEXT_ALIGN_LEFT;

	protected boolean bSSPVFileEmbedFont;

	
	public void setSSPVFileEmbedFont(boolean bSSPVFileEmbedFont ) {
		this.bSSPVFileEmbedFont = bSSPVFileEmbedFont ;
	}

	public boolean getSSPVFileEmbedFont() {
		return this.bSSPVFileEmbedFont ;
	}
	
	
	public void setFileFont(File fileFont) {
		System.out.println(fileFont);
		this.fileFont = fileFont;
	}
	
	
	public void setFont(Font font) {
		this.font = font;
	}
	
	public Font getFont() {
		return font;
	}
	
	@Override
	public String getShapeInnerXml() throws IOException {

		String sTextTransformation = "";
		switch (this.iTextTransformation ) {
		case TEXT_TRANSFORMATION_WRITE:
			sTextTransformation = "TEXT_TRANSFORMATION_WRITE";
			break;
		case TEXT_TRANSFORMATION_REMOVE:
			sTextTransformation = "TEXT_TRANSFORMATION_REMOVE";
			break;
		case TEXT_TRANSFORMATION_REPLACE:
			sTextTransformation = "TEXT_TRANSFORMATION_REPLACE";
			break;
		case TEXT_TRANSFORMATION_APPEND:
			sTextTransformation = "TEXT_TRANSFORMATION_APPEND";
			break;
		}
		
		String sTextAlign = "";
		switch (this.iTextAlign ) {
		case TEXT_ALIGN_LEFT:
			sTextAlign = "TEXT_ALIGN_LEFT";
			break;
		case TEXT_ALIGN_RIGHT:
			sTextAlign = "TEXT_ALIGN_RIGHT";
			break;
		case TEXT_ALIGN_CENTER:
			sTextAlign = "TEXT_ALIGN_CENTER";
			break;
		case TEXT_ALIGN_JUSTIFY:
			sTextAlign = "TEXT_ALIGN_JUSTIFY";
			break;
		}

		String sInnerXml = super.getShapeInnerXml();
		String sTextEncoded = this.sText;
		if(this.sTextEncoding != null)
		{
			sTextEncoded = new String(this.sText.getBytes(this.sTextEncoding));
		}
		sInnerXml += "<text>" + sTextEncoded + "</text>\n";
		if(this.fileFont != null) {
			
			
			String sFontNameXmlAttribute = "";
			try {
				// Font.TYPE1_FONT ou Font.TRUETYPE_FONT
				Font font = Font.createFont(Font.TRUETYPE_FONT, this.fileFont);
				sFontNameXmlAttribute =  " name=\"" + font.getFontName() + "\" ";
			} catch (FontFormatException e) {
				
			}
			
			String sFontFileContentBase64 = "";
			if(this.bSSPVFileEmbedFont) sFontFileContentBase64 = FileUtilBasic.encodeBase64FromFile(fileFont);
			sInnerXml += 
					"<font "
					+ sFontNameXmlAttribute
					+ " filename=\"" + this.fileFont.getName() + "\" "
					+ " embeded=\"" + this.bSSPVFileEmbedFont + "\" "
					+ " encodingType=\"" + this.encodingType + "\" "
					+ " >" 
					+ sFontFileContentBase64
					+ "</font>\n";
		}
		sInnerXml += "<fontSize>" + this.dFontSize + "</fontSize>\n";
		sInnerXml += "<textTransformation>" + sTextTransformation + "</textTransformation>\n";
		sInnerXml += "<textAlign>" + sTextAlign + "</textAlign>\n";
		return sInnerXml;
	}
	

	public void init() {
		this.shapeType = "text";
	}
	
	public PdfTextShape() {
		init();
	}

	public PdfTextShape(Object document) {
		this();
		this.document = document;
	}

	public PdfTextShape(String sText) {
		this();
		this.sText = sText;
	}
	
	
	public void setFontSize(double fontSize) {
		this.dFontSize = fontSize;
	}
	
	public double getFontSize() {
		return this.dFontSize;
	}
	
	public void setTextAlign(int textAlign) {
		this.iTextAlign = textAlign;
	}
	
	public int getTextAlign() {
		return this.iTextAlign;
	}
	
	public void setTextTransformation(int textTransformation) {
		this.iTextTransformation = textTransformation;
	}
	
	public int getTextTransformation() {
		return this.iTextTransformation;
	}
	
	public void setText(String sText) {
		this.sText= sText;
	}
	
	public String getText() {
		return this.sText;
	}
	


}
