package org.coin.util.image.shape.impl.pdfbox;

import java.awt.print.PageFormat;
import java.io.IOException;

import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPage;
import org.apache.pdfbox.pdmodel.font.PDFont;
import org.coin.util.BasicDom;
import org.coin.util.image.shape.PdfShape;
import org.coin.util.image.shape.PdfTextShape;
import org.coin.util.pdf.pdfbox.PdfBoxImageUtil;
import org.coin.util.pdf.pdfbox.PdfBoxTextUtil;
import org.coin.util.pdf.pdfbox.PdfBoxUtil;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

public class PdfBoxTextShape extends PdfTextShape {
	protected PDFont pdfont;

	public PdfBoxTextShape() {
		super();
	}

	public PdfBoxTextShape(String sText) {
		super(sText);
	}

	public void draw(int iPageNumber) throws IOException {
		
		PDPage page = PdfBoxUtil.getPage((PDDocument) document, iPageNumber - 1);
		PdfBoxTextUtil.drawTextInCell(
				this.sText, 
				this.iTextAlign, 
				this.positionX, 
				this.positionY, 
				this.width, 
				(PDDocument) this.document, 
				page,
				this.pdfont, 
				this.dFontSize,
				this.sPdfEncoding);
		
		if(this.drawAreaLimit)
		{
			PdfBoxImageUtil.drawRectangle(
					(PDDocument) this.document, 
					page, 
					this.positionX - 10, // il faut savoir pk on a cet offset !!!
					this.positionY + 30, // il faut savoir pk on a cet offset !!!
					this.positionX + this.width,
					this.positionY - this.height);
		}
	}
	
	public double getTextCellOffsetX(String sText, double dCellSizeX) throws IOException {
		double dStringWidth = PdfBoxTextUtil.getStringWidth(this.pdfont, sText) ;
		return PdfBoxTextUtil.getTextCellOffsetX(dCellSizeX, this.iTextAlign, dStringWidth);
	}


	public void setPdfont(PDFont pdfont) {
		this.pdfont = pdfont;
	}
	
	public PDFont getPdfont() {
		return pdfont;
	}


	@Override
	public int getPageOrientation(int iPageNumber) throws IOException {
		PDDocument document = (PDDocument )this.document;
		PDPage page = PdfBoxUtil.getPage((PDDocument) document, iPageNumber - 1);
		PageFormat pageFormat = PdfBoxUtil.getPageFormat(page);
		return pageFormat.getOrientation();
	}
	
	public void deserialize(Node nodeShape) throws SAXException {
		deserialize(nodeShape, this);
	}	

	public static void deserialize(
			Node nodeShape,
			PdfTextShape shapeText) 
	throws SAXException {
		shapeText.setText( BasicDom.getChildNodeValueByNodeName(nodeShape, "text"));
		String sTextTransformation = BasicDom.getChildNodeValueByNodeName(nodeShape, "textTransformation");
		if(sTextTransformation.equals("TEXT_TRANSFORMATION_WRITE")){
			shapeText.iTextTransformation = TEXT_TRANSFORMATION_WRITE;
		} else if(sTextTransformation.equals("TEXT_TRANSFORMATION_REMOVE")){
			shapeText.iTextTransformation = TEXT_TRANSFORMATION_REMOVE;
		} else if(sTextTransformation.equals("TEXT_TRANSFORMATION_REPLACE")){
			shapeText.iTextTransformation = TEXT_TRANSFORMATION_REPLACE;
		} else if(sTextTransformation.equals("TEXT_TRANSFORMATION_APPEND")){
			shapeText.iTextTransformation = TEXT_TRANSFORMATION_APPEND;
		}
		
		String sTextAlign = BasicDom.getChildNodeValueByNodeName(nodeShape, "textAlign");
		if(sTextAlign.equals("TEXT_ALIGN_CENTER")){
			shapeText.iTextAlign  = TEXT_ALIGN_CENTER;
		} else if(sTextAlign.equals("TEXT_ALIGN_LEFT")){
			shapeText.iTextAlign  = TEXT_ALIGN_LEFT;
		} else if(sTextAlign.equals("TEXT_ALIGN_RIGHT")){
			shapeText.iTextAlign  = TEXT_ALIGN_RIGHT;
		} else if(sTextAlign.equals("TEXT_ALIGN_JUSTIFY")){
			shapeText.iTextAlign  = TEXT_ALIGN_JUSTIFY;
		}
		
		/**
		 * TODO import FONT
		 */
		
		shapeText.setFontSize( BasicDom.getChildNodeValueDoubleByNodeName(nodeShape, "fontSize"));
		
		deserialize(nodeShape, (PdfShape) shapeText);
	}
	
	/**
	 * Used for all type of SSPVShape
	 * 
	 * @param nodeShape
	 * @param shapeText
	 * @throws SAXException
	 */
	public static void deserialize(
			Node nodeShape,
			PdfShape shapeText) 
	throws SAXException {
		shapeText.setName( BasicDom.getChildNodeValueByNodeName(nodeShape, "name"));
		shapeText.setPageNumberList( BasicDom.getChildNodeValueByNodeName(nodeShape, "pageNumberList"));
		shapeText.setPositionX( BasicDom.getChildNodeValueDoubleByNodeName(nodeShape, "positionX"));
		shapeText.setPositionY( BasicDom.getChildNodeValueDoubleByNodeName(nodeShape, "positionY"));
		shapeText.setWidth( BasicDom.getChildNodeValueDoubleByNodeName(nodeShape, "width"));
		shapeText.setHeight( BasicDom.getChildNodeValueDoubleByNodeName(nodeShape, "height"));
	}



}
